@@ -183,13 +183,12 @@ sub connect ($$;$$) {
 	    $this->{$var} = $attr{$var};
 	}
     }
+    $this->SUPER::STORE('Active' => 1);
 
     $this;
 }
 
 
-sub disconnect_all { }
-
 sub DESTROY { undef }
 
 
@@ -234,6 +233,7 @@ sub AUTOLOAD {
 	  'type' => $type,
 	  'h' => "DBI::_::$type"
 	);
+    local $SIG{__DIE__} = 'DEFAULT';
     my $method_code = UNIVERSAL::can($expand{'h'}, $method) ?
 	q/package ~class~;
           sub ~method~ {
@@ -269,6 +269,7 @@ sub disconnect ($) {
     # to let people trade safety for speed if they need to.
     undef $dbh->{'proxy_dbh'};    # Bug in Perl 5.004; would prefer delete
     undef $dbh->{'proxy_client'};
+    $this->SUPER::STORE('Active' => 0);
     1;
 }
 
@@ -283,6 +284,7 @@ sub STORE ($$$) {
     }
 
     if ($type eq 'remote'  ||  $type eq 'cached') {
+        local $SIG{__DIE__} = 'DEFAULT';
 	my $result = eval { $dbh->{'proxy_dbh'}->STORE($attr => $val) };
 	return DBD::Proxy::proxy_set_err($dbh, $@) if $@; # returns undef
 	$dbh->{$attr} = $val if $type eq 'cached';
@@ -302,6 +304,7 @@ sub FETCH ($$) {
 
     return $dbh->SUPER::FETCH($attr) unless $type eq 'remote';
 
+    local $SIG{__DIE__} = 'DEFAULT';
     my $result = eval { $dbh->{'proxy_dbh'}->FETCH($attr) };
     return DBD::Proxy::proxy_set_err($dbh, $@) if $@;
     return $result;
@@ -320,6 +323,7 @@ sub prepare ($$;$) {
     if ( $proto_ver > 1 ) {
       $sth->{'proxy_attr_cache'} = {cache_filled => 0};
       my $rdbh = $dbh->{'proxy_dbh'};
+      local $SIG{__DIE__} = 'DEFAULT';
       my $rsth = eval { $rdbh->prepare($sth->{'Statement'}, $sth->{'proxy_attr'}, undef, $proto_ver) };
       return DBD::Proxy::proxy_set_err($sth, $@) if $@;
       return DBD::Proxy::proxy_set_err($sth, "Constructor didn't return a handle: $rsth")
@@ -355,7 +359,7 @@ sub quote {
     #    $dbh->{'proxy_quote'} = 'backslash_escaped';
     # for example.
     # Jochen
-
+    local $SIG{__DIE__} = 'DEFAULT';
     my $result = eval { $dbh->{'proxy_dbh'}->quote(@_) };
     return DBD::Proxy::proxy_set_err($dbh, $@) if $@;
     return $result;
@@ -365,6 +369,7 @@ sub table_info {
     my $dbh = shift;
     my $rdbh = $dbh->{'proxy_dbh'};
     #warn "table_info(@_)";
+    local $SIG{__DIE__} = 'DEFAULT';
     my($numFields, $names, $types, @rows) = eval { $rdbh->table_info(@_) };
     return DBD::Proxy::proxy_set_err($dbh, $@) if $@;
     my $sth = DBI::_new_sth($dbh, {
@@ -395,6 +400,7 @@ sub tables {
 
 sub type_info_all {
     my $dbh = shift;
+    local $SIG{__DIE__} = 'DEFAULT';
     my $result = eval { $dbh->{'proxy_dbh'}->type_info_all(@_) };
     return DBD::Proxy::proxy_set_err($dbh, $@) if $@;
     return $result;
@@ -450,6 +456,7 @@ sub execute ($@) {
 
     my ($numRows, @outData);
 
+    local $SIG{__DIE__} = 'DEFAULT';
     if ( $proto_ver > 1 ) {
       ($numRows, @outData) = eval { $rsth->execute($params, $proto_ver) };
       return DBD::Proxy::proxy_set_err($sth, $@) if $@;
@@ -467,12 +474,14 @@ sub execute ($@) {
 	$sth->SUPER::STORE('NUM_OF_PARAMS' => $numParams);
       }
 
-    } else {
+    }
+    else {
       if ($rsth) {
 	($numRows, @outData) = eval { $rsth->execute($params, $proto_ver) };
 	return DBD::Proxy::proxy_set_err($sth, $@) if $@;
 
-      } else {
+      }
+      else {
 	my $rdbh = $dbh->{'proxy_dbh'};
 	
 	# Legacy prepare is actually prepare + first execute on the server.
@@ -519,7 +528,7 @@ sub fetch ($) {
 
     my $data = $sth->{'proxy_data'};
 
-    if(!$data  ||  !@$data) {
+    if(!$data || !@$data) {
 	return undef unless $sth->SUPER::FETCH('Active');
 
 	my $rsth = $sth->{'proxy_sth'};
@@ -527,6 +536,7 @@ sub fetch ($) {
 	    die "Attempt to fetch row without execute";
 	}
 	my $num_rows = $sth->FETCH('RowCacheSize') || 20;
+	local $SIG{__DIE__} = 'DEFAULT';
 	my @rows = eval { $rsth->fetch($num_rows) };
 	return DBD::Proxy::proxy_set_err($sth, $@) if $@;
 	unless (@rows == $num_rows) {
@@ -539,7 +549,7 @@ sub fetch ($) {
     }
     my $row = shift @$data;
 
-	$sth->SUPER::STORE(Active => 0) if ( $sth->{proxy_cache_only} and !@$data );
+    $sth->SUPER::STORE(Active => 0) if ( $sth->{proxy_cache_only} and !@$data );
     return $sth->_set_fbav($row);
 }
 *fetchrow_arrayref = \&fetch;
@@ -559,6 +569,7 @@ sub finish ($) {
  	? $sth->{'proxy_no_finish'}
 	: $sth->FETCH('Database')->{'proxy_no_finish'};
     unless ($no_finish) {
+        local $SIG{__DIE__} = 'DEFAULT';
 	my $result = eval { $rsth->finish() };
 	return DBD::Proxy::proxy_set_err($sth, $@) if $@;
 	return $result;
@@ -581,6 +592,7 @@ sub STORE ($$$) {
 
     if ($type eq 'remote') {
 	my $rsth = $sth->{'proxy_sth'}  or  return undef;
+        local $SIG{__DIE__} = 'DEFAULT';
 	my $result = eval { $rsth->STORE($attr => $val) };
 	return DBD::Proxy::proxy_set_err($sth, $@) if ($@);
 	return $result;
@@ -610,6 +622,7 @@ sub FETCH ($$) {
 
     if ($type ne 'local') {
 	my $rsth = $sth->{'proxy_sth'}  or  return undef;
+        local $SIG{__DIE__} = 'DEFAULT';
 	my $result = eval { $rsth->FETCH($attr) };
 	return DBD::Proxy::proxy_set_err($sth, $@) if $@;
 	return $result;
