@@ -1,26 +1,25 @@
 package DBI::DBD::Metadata;
 
-# $Id: Metadata.pm 14213 2010-06-30 19:29:18Z Martin $
+# $Id: Metadata.pm,v 1.1 2003/02/26 22:28:54 timbo Exp $
 #
-# Copyright (c) 1997-2003 Jonathan Leffler, Jochen Wiedmann,
+# Copyright (c) 1997-2002 Jonathan Leffler, Jochen Wiedmann,
 # Steffen Goeldner and Tim Bunce
 #
 # You may distribute under the terms of either the GNU General Public
 # License or the Artistic License, as specified in the Perl README file.
 
 use strict;
-
 use Exporter ();
 use Carp;
 
 use DBI;
 use DBI::Const::GetInfoType qw(%GetInfoType);
 
+
 our @ISA = qw(Exporter);
 our @EXPORT = qw(write_getinfo_pm write_typeinfo_pm);
 
-our $VERSION = "2.014214";
-
+our $VERSION = sprintf("%d.%02d", q$Revision: 1.1 $ =~ /(\d+)\.(\d+)/);
 
 =head1 NAME
 
@@ -32,13 +31,13 @@ The idea is to extract metadata information from a good quality
 ODBC driver and use it to generate code and data to use in your own
 DBI driver for the same database.
 
-To generate code to support the get_info method:
+For generating code to support the get_info method:
 
   perl -MDBI::DBD::Metadata -e "write_getinfo_pm('dbi:ODBC:dsn-name','user','pass','Driver')"
 
   perl -MDBI::DBD::Metadata -e write_getinfo_pm dbi:ODBC:foo_db username password Driver
 
-To generate code to support the type_info method:
+For generating code to support the type_info method:
 
   perl -MDBI::DBD::Metadata -e "write_typeinfo_pm('dbi:ODBC:dsn-name','user','pass','Driver')"
 
@@ -47,7 +46,7 @@ To generate code to support the type_info method:
 Where C<dbi:ODBC:dsn-name> is the connection to use to extract the
 data, and C<Driver> is the name of the driver you want the code
 generated for (the driver name gets embedded into the output in
-numerous places).
+many places).
 
 =head1 Generating a GetInfo package for a driver
 
@@ -57,13 +56,13 @@ DBD::Driver::GetInfo package on standard output.
 This method generates a DBD::Driver::GetInfo package from the data
 source you specified in the parameter list or in the environment
 variable DBI_DSN.
-DBD::Driver::GetInfo should help a DBD author implement the DBI
+DBD::Driver::GetInfo should help a DBD author implementing the DBI
 get_info() method.
-Because you are just creating this package, it is very unlikely that
+Because you are just creating this package, it's very unlikely that
 DBD::Driver already provides a good implementation for get_info().
 Thus you will probably connect via DBD::ODBC.
 
-Once you are sure that it is producing reasonably sane data, you should
+Once you are sure that it is producing semi-sane data, you would
 typically redirect the standard output to lib/DBD/Driver/GetInfo.pm, and
 then hand edit the result.
 Do not forget to update your Makefile.PL and MANIFEST to include this as
@@ -78,7 +77,7 @@ The generator assumes that most values are static and places these
 values directly in the %info hash.
 A few examples show the use of CODE references and the implementation
 via subroutines.
-It is very likely that you will have to write additional subroutines for
+It is very likely that you have to write additional subroutines for
 values depending on the session state or server version, e.g.
 SQL_DBMS_VER.
 
@@ -137,7 +136,7 @@ use DBD::${driver};
 
 my \$sql_driver = '${driver}';
 my \$sql_ver_fmt = '%02d.%02d.%04d';   # ODBC version string: ##.##.#####
-my \$sql_driver_ver = sprintf \$sql_ver_fmt, split (/\\./, \$DBD::${driver}::VERSION);
+my \$sql_driver_ver = sprintf \$sql_ver_fmt, split (/\./, \$DBD::${driver}::VERSION);
 PERL
 
 my $kw_map = 0;
@@ -167,9 +166,8 @@ sub sql_data_source_name {
 
 sub sql_user_name {
     my $dbh = shift;
-    # CURRENT_USER is a non-standard attribute, probably undef
-    # Username is a standard DBI attribute
-    return $dbh->{CURRENT_USER} || $dbh->{Username};
+    # Non-standard attribute
+    return $dbh->{CURRENT_USER};
 }
 
 PERL
@@ -185,9 +183,6 @@ PERL
         elsif ($key eq 'SQL_KEYWORDS') {
             $val = ($kw_map) ? '\&sql_keywords' : 'undef';
         }
-        elsif ($key eq 'SQL_DRIVER_NAME') {
-            $val = "\$INC{'DBD/$driver.pm'}";
-        }
         elsif ($key eq 'SQL_DRIVER_VER') {
             $val = '$sql_driver_ver';
         }
@@ -226,7 +221,7 @@ where the odbc_dsn is a DSN for one of the driver's databases.
 The user and pass parameters are the other optional connection
 parameters that will be provided to the DBI connect method.
 
-Once you are sure that it is producing reasonably sane data, you should
+Once you are sure that it is producing semi-sane data, you would
 typically redirect the standard output to lib/DBD/Driver/TypeInfo.pm,
 and then hand edit the result if necessary.
 Do not forget to update your Makefile.PL and MANIFEST to include this as
@@ -243,19 +238,19 @@ A possible implementation of DBD::Driver::type_info_all() may look like:
   sub type_info_all {
     my ($dbh) = @_;
     require DBD::Driver::TypeInfo;
-    return [ @$DBD::Driver::TypeInfo::type_info_all ];
+    return $DBD::Driver::TypeInfo::type_info_all;
   }
 
 Please replace Driver (or "<foo>") with the name of your driver.
-Note that this stub function is generated for you by the write_typeinfo_pm
+Note that this stub function is generated for you by the write_typeinfo
 function, but you must manually transfer the code to Driver.pm.
 
 =cut
 
 
 # These two are used by fmt_value...
-my %dbi_inv;
-my %sql_type_inv;
+our %dbi_inv;
+our %sql_type_inv;
 
 #-DEBUGGING-#
 #sub print_hash
@@ -307,9 +302,9 @@ sub fmt_value
     return $val;
 }
 
-sub write_typeinfo_pm
+sub write_typeinfo
 {
-    my ($dsn, $user, $pass, $driver) = @_ ? @_ : @ARGV;
+    my ($driver, $dsn, $user, $pass) = @_ ? @_ : @ARGV;
     my $dbh = DBI->connect($dsn, $user, $pass, {AutoCommit=>1, RaiseError=>1});
     $driver = "<foo>" unless defined $driver;
 
@@ -318,7 +313,7 @@ sub write_typeinfo_pm
 # Transfer this to ${driver}.pm
 
 # The type_info_all function was automatically generated by
-# DBI::DBD::Metadata::write_typeinfo_pm v$DBI::DBD::Metadata::VERSION.
+# DBI::DBD::Metadata::write_typeinfo v$DBI::DBD::Metadata::VERSION.
 
 package DBD::${driver}::db;         # This line can be removed once transferred.
 
@@ -326,14 +321,14 @@ package DBD::${driver}::db;         # This line can be removed once transferred.
     {
         my (\$dbh) = \@_;
         require DBD::${driver}::TypeInfo;
-        return [ \@\$DBD::${driver}::TypeInfo::type_info_all ];
+        return \$DBD::${driver}::TypeInfo::type_info_all;
     }
 
 # Transfer this to lib/DBD/${driver}/TypeInfo.pm.
 # Don't forget to add version and intellectual property control information.
 
 # The \%type_info_all hash was automatically generated by
-# DBI::DBD::Metadata::write_typeinfo_pm v$DBI::DBD::Metadata::VERSION.
+# DBI::DBD::Metadata::write_typeinfo v$DBI::DBD::Metadata::VERSION.
 
 package DBD::${driver}::TypeInfo;
 
@@ -342,7 +337,7 @@ package DBD::${driver}::TypeInfo;
     require DynaLoader;
     \@ISA = qw(Exporter DynaLoader);
     \@EXPORT = qw(type_info_all);
-    use DBI qw(:sql_types);
+    use DBI (:sql_types);
 
 PERL
 
@@ -397,7 +392,7 @@ PERL
     my $fmt = "            \%-${maxlen}s => \%2d,\n";
     my $numkey = 0;
     my $maxkey = 0;
-    print "    \$type_info_all = [\n        {\n";
+    print "    our \$type_info_all = [\n        {\n";
     foreach my $i (sort { $a <=> $b } keys %dbi_inv)
     {
         printf($fmt, $dbi_inv{$i}, $i);
@@ -411,7 +406,7 @@ PERL
 
     my $h = $dbh->type_info_all;
     my @tia = @$h;
-    my %odbc_map = map { uc $_ => $tia[0]->{$_} } keys %{$tia[0]};
+    my %odbc_map = %{$tia[0]};
     shift @tia;     # Remove the mapping reference.
     my $numtyp = $#tia;
 
@@ -486,7 +481,7 @@ __END__
 
 Jonathan Leffler <jleffler@us.ibm.com> (previously <jleffler@informix.com>),
 Jochen Wiedmann <joe@ispsoft.de>,
-Steffen Goeldner <sgoeldner@cpan.org>,
+Steffen Goeldner <s.goeldner@eurodata.de>,
 and Tim Bunce <dbi-users@perl.org>.
 
 =cut
