@@ -1,9 +1,10 @@
 ########################################################################
-package DBI;
-
+package		# hide from PAUSE
+	DBI;
+# vim: ts=8:sw=4
 ########################################################################
 #
-# Copyright (c) 2002  Tim Bunce  Ireland.
+# Copyright (c) 2002,2003  Tim Bunce  Ireland.
 #
 # See COPYRIGHT section in DBI.pm for usage and distribution rights.
 #
@@ -17,14 +18,13 @@ package DBI;
 
 # TODO:
 #	recheck code against DBI
-#	support TraceLevel and other attributes
 
 use strict;
 use Carp;
 require Symbol;
 
 $DBI::PurePerl = $ENV{DBI_PUREPERL} || 1;
-$DBI::PurePerl::VERSION = substr(q$Revision: 1.10 $, 10);
+$DBI::PurePerl::VERSION = sprintf "%d.%02d", '$Revision: 1.92 $ ' =~ /(\d+)\.(\d+)/;
 $DBI::neat_maxlen ||= 400;
 
 $DBI::tfh = Symbol::gensym();
@@ -94,49 +94,60 @@ use constant SQL_WVARCHAR => (-9);
 use constant IMA_HAS_USAGE	=> 0x0001; #/* check parameter usage	*/
 use constant IMA_FUNC_REDIRECT	=> 0x0002; #/* is $h->func(..., "method")*/
 use constant IMA_KEEP_ERR	=> 0x0004; #/* don't reset err & errstr	*/
-use constant IMA_spare		=> 0x0008; #/* */
+use constant IMA_KEEP_ERR_SUB	=> 0x0008; #/*  '' if in nested call */
 use constant IMA_NO_TAINT_IN   	=> 0x0010; #/* don't check for tainted args*/
 use constant IMA_NO_TAINT_OUT   => 0x0020; #/* don't taint results	*/
 use constant IMA_COPY_STMT   	=> 0x0040; #/* copy sth Statement to dbh */
 use constant IMA_END_WORK	=> 0x0080; #/* set on commit & rollback	*/
 use constant IMA_STUB		=> 0x0100; #/* donothing eg $dbh->connected */
+#define IMA_CLEAR_STMT             0x0200  /* clear Statement before call  */
+#define IMA_PROF_EMPTY_STMT        0x0400  /* profile as empty Statement   */
+use constant IMA_NOT_FOUND_OKAY	=> 0x0800; #/* not error if not found */
 
-my %is_valid_attribute = map {$_ =>1 } qw(
+my %is_flag_attribute = map {$_ =>1 } qw(
 	Active
-	ActiveKids
-	Attribution
 	AutoCommit
-	BegunWork
-	CachedKids
 	ChopBlanks
 	CompatMode
+	Taint
+	TaintIn
+	TaintOut
+	InactiveDestroy
+	LongTruncOk
+	MultiThread
+	PrintError
+	RaiseError
+	ShowErrorStatement
+	Warn
+);
+my %is_valid_attribute = map {$_ =>1 } (keys %is_flag_attribute, qw(
+	ActiveKids
+	Attribution
+	BegunWork
+	CachedKids
+	CursorName
 	Database
 	DebugDispatch
 	Driver
 	FetchHashKeyName
 	HandleError
 	ImplementorClass
-	InactiveDestroy
 	Kids
 	LongReadLen
-	LongTruncOk
-	MultiThread
 	NAME NAME_uc NAME_lc NAME_uc_hash NAME_lc_hash
 	NUM_OF_FIELDS
 	NUM_OF_PARAMS
 	Name
-	PrintError
+	ParamValues
+	Profile
 	Provider
-	RaiseError
 	RootClass
+	RowsInCache
 	RowCacheSize
-	ShowErrorStatement
 	Statement
-	Taint
 	TraceLevel
 	Version
-	Warn
-);
+));
 
 sub valid_attribute {
     my $attr = shift;
@@ -164,9 +175,20 @@ sub  _install_method {
     my $bitmask = $param_hash->{'O'} || 0;
     my @pre_call_frag;
 
+    push @pre_call_frag, q{
+	return if $h_inner; # ignore DESTROY for outer handle
+    } if $method_name eq 'DESTROY';
+
+    push @pre_call_frag, q{
+	# copy err/errstr/state up to driver so $DBI::err etc still work
+	if ($h->{err} and my $drh = $h->{Driver}) {
+	    $drh->{$_} = $h->{$_} for ('err','errstr','state');
+	}
+    } if $method eq 'DBI::db::DESTROY';
+
     push @pre_call_frag, q{
 	return $h->{$_[0]} if exists $h->{$_[0]};
-    } if $method_name eq 'FETCH';
+    } if $method_name eq 'FETCH' && !exists $ENV{DBI_TRACE};
 
     push @pre_call_frag, "return;"
 	if IMA_STUB & $bitmask;
@@ -181,10 +203,29 @@ sub  _install_method {
 	$dbh->{Statement} = $h->{Statement} if $dbh;
     } if IMA_COPY_STMT & $bitmask;
 
-    push @pre_call_frag, q{
-	$h->{err} = 0;
-	$h->{errstr} = $DBI::state = '';
-    } unless IMA_KEEP_ERR & $bitmask;	# see above
+    if (IMA_KEEP_ERR & $bitmask) {
+	push @pre_call_frag, q{
+	    my $keep_error = 1;
+	};
+    }
+    elsif (IMA_KEEP_ERR_SUB & $bitmask) {
+	push @pre_call_frag, q{
+	    my $keep_error = $h->{_parent}->{_call_depth};
+	    unless ($keep_error) {
+		#warn "$method_name cleared err";
+		$h->{err} = 0;
+		$h->{errstr} = $DBI::state = '';
+	    };
+	};
+    }
+    else {
+	push @pre_call_frag, q{
+	    my $keep_error;
+	    #warn "$method_name cleared err";
+	    $h->{err} = 0;
+	    $h->{errstr} = $DBI::state = '';
+	};
+    }
 
     push @pre_call_frag, q{
         if ($DBI::dbi_debug >= 2) {
@@ -198,7 +239,6 @@ sub  _install_method {
         $h->{'_last_method'} = $method_name;
     } unless exists $DBI::last_method_except{$method_name};
 
-
     # --- post method call code fragments ---
     my @post_call_frag;
 
@@ -213,28 +253,40 @@ sub  _install_method {
     } if exists $ENV{DBI_TRACE}; # note use of exists
 
     push @post_call_frag, q{
-	if ($h->{BegunWork}) {;
+	if ($h->{BegunWork}) {
 	    $h->{BegunWork}  = 0;
 	    $h->{AutoCommit} = 1;
 	}
     } if IMA_END_WORK & $bitmask;
 
     push @post_call_frag, q{
-        if ($h->{err} && $call_depth <= 1) {
+        if ($h->{err} && !$keep_error && $call_depth <= 1 && !$h->{_parent}{_call_depth}) {
             my $last = ($DBI::last_method_except{$method_name})
 		? ($h->{'_last_method'}||$method_name) : $method_name;
             my $errstr = $h->{errstr} || $DBI::errstr || $DBI::err || '';
-            my $msg = sprintf "%s %s failed: %s\n", $imp, $last, $errstr;
-            if ($h->{'ShowErrorStatement'} && $h->{'Statement'}) {
-               $msg .= " for [\"".$h->{'Statement'}."\"]";
+            my $msg = sprintf "%s %s failed: %s", $imp, $last, $errstr;
+            if ($h->{'ShowErrorStatement'} and my $Statement = $h->{Statement}) {
+		$msg .= ' for [``' . $Statement . "''";
+		my $ParamValues = $h->FETCH('ParamValues');
+		if ($ParamValues) {
+		    my $pv_idx = 0;
+		    $msg .= " with params: ";
+		    while ( my($k,$v) = each %$ParamValues ) {
+			$msg .= sprintf "%s%s=%s", ($pv_idx++==0) ? "" : ", ", $k, DBI::neat($v);
+		    }
+		}
+		$msg .= "]";
 	    }
+	    $msg .= "\n";
             my $do_croak = 1;
             if (my $subsub = $h->{'HandleError'}) {
 		$do_croak = 0 if &$subsub($msg,$h,$ret[0]);
             }
 	    if ($do_croak) {
+		printf $DBI::tfh "    $method_name has failed ($h->{PrintError},$h->{RaiseError})\n"
+			if $DBI::dbi_debug >= 4;
   	        carp  $msg if $h->{PrintError};
-	        croak $msg if $h->{RaiseError};
+	        die $msg if $h->{RaiseError};
 	    }
 	}
     } unless IMA_KEEP_ERR & $bitmask;
@@ -244,11 +296,6 @@ sub  _install_method {
       sub {
         my $h = shift;
 	my $h_inner = tied(%$h);
-
-	]
-	. ($method_name eq 'DESTROY' ? "return if \$h_inner;\n" : "")
-	. q[
-
 	$h = $h_inner if $h_inner;
 	# XXX this eval isn't good because it overwrites $@
         my $imp = eval { $h->{"ImplementorClass"} } or return; # probably global destruction
@@ -258,10 +305,15 @@ sub  _install_method {
 	my $call_depth = $h->{'_call_depth'} + 1;
 	local ($h->{'_call_depth'}) = $call_depth;
 
-        my $sub = $imp->can($method_name) or croak "Can't find $method_name method for $h";
-
 	my @ret;
-        (wantarray) ? (@ret = &$sub($h,@_)) : (@ret = scalar &$sub($h,@_));
+        my $sub = $imp->can($method_name);
+	if ($sub) {
+	    (wantarray) ? (@ret = &$sub($h,@_)) : (@ret = scalar &$sub($h,@_));
+	}
+	else {
+	    croak "Can't find DBI method $method_name for $h"
+		if ] . ((IMA_NOT_FOUND_OKAY & $bitmask) ? 0 : 1) . q[;
+	}
 
 	] . join("\n", '', @post_call_frag, '') . q[
 
@@ -269,12 +321,13 @@ sub  _install_method {
       }
     ];
     no strict qw(refs);
-    my $code_ref = eval $method_code;
-    *$method = eval qq{#line 1 "$method"\n$method_code};
+    my $code_ref = eval qq{#line 1 "$method"\n$method_code};
+    warn "$@\n$method_code\n" if $@;
     die "$@\n$method_code\n" if $@;
-    if (0 && $method =~ 'set_err') { # debuging tool
+    *$method = $code_ref;
+    if (0 && $method =~ 'DESTROY') { # debuging tool
 	my $l=0; # show line-numbered code for method
-	warn join("\n", map { ++$l.": $_" } split/\n/,$method_code);
+	warn "*$method = ".join("\n", map { ++$l.": $_" } split/\n/,$method_code);
     }
 }
 
@@ -282,27 +335,37 @@ sub _setup_handle {
     my($h, $imp_class, $parent, $imp_data) = @_;
     my $h_inner = tied(%$h) || $h;
     warn "\n_setup_handle(@_)" if $DBI::dbi_debug >= 4;
-    if (ref($parent) =~ /^[^:]+::dr/){
-        #$h_inner->{$_} ||= $parent->{$_} foreach (qw(Name Version Attribution));
-        $h_inner->{Driver} = $parent;
-    }
-    $h_inner->{"_call_depth"} = 0;
     $h_inner->{"imp_data"} = $imp_data;
     $h_inner->{"ImplementorClass"} = $imp_class;
     $h_inner->{"Kids"} = $h_inner->{"ActiveKids"} = 0;	# XXX not maintained
-    $h_inner->{"Active"} = 1;
-    $h_inner->{"FetchHashKeyName"} ||= $parent->{"FetchHashKeyName"} || 'NAME';
-    $h_inner->{"Taint"}= 0 unless defined $h_inner->{"Taint"};
-    $h_inner->{"Warn"} = 1 unless defined $h_inner->{"Warn"};
-    if (ref($parent) =~ /^[^:]+::db/){
-        $h_inner->{'Database'}    = $parent;
-        $h_inner->{'RaiseError'}  = $parent->{'RaiseError'};
-        $h_inner->{'PrintError'}  = $parent->{'PrintError'};
-        $h_inner->{'HandleError'} = $parent->{'HandleError'};
-        $parent->{Statement} = $h_inner->{Statement};
-	#use Data::Dumper; $Data::Dumper::Indent=1;
-	#warn Dumper([ "$h_inner, $parent", $h_inner->{Database}]);
+    if ($parent) {
+	foreach (qw(
+	    RaiseError PrintError HandleError
+	    Warn LongTruncOk ChopBlanks AutoCommit
+	    ShowErrorStatement FetchHashKeyName LongReadLen CompatMode
+	)) {
+	    $h_inner->{$_} = $parent->{$_}
+		if exists $parent->{$_} && !exists $h_inner->{$_};
+	}
+	if (ref($parent) =~ /::db$/) {
+	    $h_inner->{Database} = $parent;
+	    $parent->{Statement} = $h_inner->{Statement};
+	}
+	elsif (ref($parent) =~ /::dr$/){
+	    $h_inner->{Driver} = $parent;
+	}
+	$h_inner->{_parent} = $parent;
     }
+    else {	# setting up a driver handle
+        $h_inner->{Warn}		= 1;
+        $h_inner->{AutoCommit}		= 1;
+        $h_inner->{TraceLevel}		= 0;
+        $h_inner->{CompatMode}		= (1==0);
+	$h_inner->{FetchHashKeyName}	||= 'NAME';
+	$h_inner->{LongReadLen}		||= 80;
+    }
+    $h_inner->{"_call_depth"} = 0;
+    $h_inner->{"Active"} = 1;
 }
 sub constant {
     warn "constant @_"; return;
@@ -324,10 +387,14 @@ sub trace {
 sub _set_trace_file {
     my ($file) = @_;
     return unless defined $file;
-    unless ($file) {
+    if (!$file || $file eq 'STDERR') {
 	open $DBI::tfh, ">&STDERR" or warn "Can't dup STDERR: $!";
 	return 1;
     }
+    if ($file eq 'STDOUT') {
+	open $DBI::tfh, ">&STDOUT" or warn "Can't dup STDOUT: $!";
+	return 1;
+    }
     open $DBI::tfh, ">>$file" or carp "Can't open $file: $!";
     select((select($DBI::tfh), $| = 1)[0]);
     return 1;
@@ -414,24 +481,18 @@ sub neat {
     return "'$v'";
 }
 
-package DBI::var;              # ============ DBI::var
+package
+	DBI::var;
 
 sub FETCH {
     my($key)=shift;
     return $DBI::err     if $$key eq '*err';
     return $DBI::errstr  if $$key eq '&errstr';
-    if ($$key eq '"state'){
-die; # XXX
-        my $state = $DBI::state;
-        return $state if $state;
-        return '' unless defined $state;
-	$state= ($DBI::err) ? "S1000" : "00000" unless $state;
-        return $state;
-    }
     Carp::croak("FETCH $key not supported when using DBI::PurePerl");
 }
 
-package DBD::_::common;		# ============ DBD::_::common
+package
+	DBD::_::common;
 
 sub trace {	# XXX should set per-handle level, not global
     my ($h, $level, $file) = @_;
@@ -440,7 +501,7 @@ sub trace {	# XXX should set per-handle level, not global
 	$DBI::dbi_debug = $level;
 	printf $DBI::tfh
             "    %s trace level set to %d in DBI $DBI::VERSION (PurePerl)\n",
-	    $h, $level if $file;
+	    $h, $level if $level;
     }
     _set_trace_file($file) if defined $file;
     return $old_level;
@@ -449,7 +510,10 @@ sub trace {	# XXX should set per-handle level, not global
 
 sub FETCH {
     my($h,$key)= @_;
-    if (!$h->{$key} and $key =~ /^NAME_.c$/) {
+    my $v = $h->{$key};
+#warn ((exists $h->{$key}) ? "$key=$v\n" : "$key NONEXISTANT\n");
+    return $v if defined $v;
+    if ($key =~ /^NAME_.c$/) {
         my $cols = $h->FETCH('NAME');
         return undef unless $cols;
         my @lcols = map { lc $_ } @$cols;
@@ -458,7 +522,7 @@ sub FETCH {
         $h->STORE('NAME_uc',\@ucols);
         return $h->FETCH($key);
     }
-    if (!$h->{$key} and $key =~ /^NAME.*_hash$/) {
+    if ($key =~ /^NAME.*_hash$/) {
         my $i=0;
         for my $c(@{$h->FETCH('NAME')}) {
             $h->{'NAME_hash'}->{$c}    = $i;
@@ -466,11 +530,16 @@ sub FETCH {
             $h->{'NAME_uc_hash'}->{"\U$c"} = $i;
             $i++;
         }
+        return $h->{$key};
     }
-    my $v = $h->{$key};
     if (!defined $v && !exists $h->{$key}) {
-	Carp::croak( sprintf "Can't get %s->{%s}: unrecognised attribute (@{[ %$h ]})",$h,$key)
-	    if !$is_valid_attribute{$key} and $key =~ m/^[A-Z]/;
+	return ($h->FETCH('TaintIn') && $h->FETCH('TaintOut')) if $key eq'Taint';
+	return (1==0) if $is_flag_attribute{$key}; # return perl-style sv_no, not undef
+	return $DBI::dbi_debug if $key eq 'TraceLevel';
+	if (!$is_valid_attribute{$key} and $key =~ m/^[A-Z]/) {
+	    local $^W; # hide undef warnings
+	    Carp::croak( sprintf "Can't get %s->{%s}: unrecognised attribute (@{[ %$h ]})",$h,$key )
+	}
     }
     return $v;
 }
@@ -481,18 +550,23 @@ sub STORE {
 	    unless $value == -900 || $value == -901;
 	$value = ($value == -901);
     }
+    elsif ($key =~ /^Taint/ ) {
+	Carp::croak(sprintf "Can't set %s->{%s}: Taint mode not supported by DBI::PurePerl",$h,$key);
+    }
+    elsif ($key eq 'TraceLevel') {
+	$DBI::dbi_debug = $value;
+	return 1;
+    }
     elsif (!$is_valid_attribute{$key} && $key =~ /^[A-Z]/ && !exists $h->{$key}) {
        Carp::croak(sprintf "Can't set %s->{%s}: unrecognised attribute or invalid value %s",
 	    $h,$key,$value);
     }
-    $h->{$key} = $value;
+    $h->{$key} = $is_flag_attribute{$key} ? !!$value : $value;
     return 1;
 }
 sub err {
     my $h = shift;
-    # XXX need to be shared between dbh and sth?
-    my $err = $h->{'err'} || $h->{'errstr'};
-    return $err;
+    return $h->{'err'} || $h->{'errstr'};
 }
 sub errstr {
     my $h = shift;
@@ -502,25 +576,24 @@ sub errstr {
 sub state {
     return shift->{state};
 }
-sub event { }
 sub set_err {
     my ($h, $errnum,$msg,$state, $method, $rv) = @_;
     $h = tied(%$h) || $h; # for code that calls $h->DBI::set_err(...)
     $msg = $errnum unless defined $msg;
-    $h->{'_last_method'} = $method if $method;
+    $h->{'_last_method'} = $method;
+    $h->{err}    = $DBI::err    = $errnum;
     $h->{errstr} = $DBI::errstr = $msg;
     $h->{state}  = $DBI::state  = (defined $state) ? ($state eq "00000" ? "" : $state) : ($errnum ? "S1000" : "");
-    $h->{err}    = $DBI::err    = $errnum;
-    if (my $dbh = $h->{Database}) {
-	$dbh->{errstr} = $msg;
-	$dbh->{err} = $errnum;
+    if (my $p = $h->{Database}) { # just sth->dbh, not dbh->drh (see ::db::DESTROY)
+	$p->{err}    = $errnum;
+	$p->{errstr} = $msg;
+	$p->{state}  = $DBI::state;
     }
     return $rv; # usually undef
 }
 sub trace_msg {
     my ($h, $msg, $minlevel)=@_;
     $minlevel = 1 unless defined $minlevel;
-    #my $curlevel = (ref $h) ? $h->{TraceLevel} : 0;	# XXX
     my $curlevel = $DBI::dbi_debug; # XXX if $DBI::dbi_debug > $curlevel;
     return if $curlevel < $minlevel;
     print $DBI::tfh $msg;
@@ -535,7 +608,8 @@ sub rows {
 sub DESTROY {
 }
 
-package DBD::_::st;		# ============ DBD::_::st
+package
+	DBD::_::st;
 
 sub fetchrow_arrayref	{
     my $h = shift;
@@ -719,6 +793,9 @@ and the files are installed automatically?)
 
 =head2 Attributes
 
+Boolean attributes still return boolean values but the actual values
+used may be different, i.e., 0 or undef instead of an empty string.
+
 Some handle attributes are either not supported or have very limited
 functionality:
 
@@ -726,7 +803,8 @@ functionality:
   InactiveDestroy
   Kids
   Taint
-  TraceLevel
+  TaintIn
+  TaintOut
 
 (and probably others)
 
